@@ -1,8 +1,9 @@
-from typing import List, Optional, Tuple, Dict
+from typing import List, Optional, Tuple, Dict, Callable
 from functools import reduce
+from pathlib import Path
 import re
 from .utils import maybe_is_text, maybe_is_truncated
-from .qaprompts import summary_prompt, qa_prompt, edit_prompt
+from .qaprompts import summary_prompt, qa_prompt, edit_prompt, search_prompt
 from dataclasses import dataclass
 from .readers import read_doc
 from langchain.vectorstores import FAISS
@@ -38,6 +39,8 @@ class Docs:
         chunk_size_limit: int = 3000,
         llm: Optional[LLM] = None,
         summary_llm: Optional[LLM] = None,
+        name: str = "default",
+        index_path: Optional[Path] = None,
     ) -> None:
         """Initialize the collection of documents.
 
@@ -47,6 +50,8 @@ class Docs:
             chunk_size_limit: The maximum number of characters to use for a single chunk of text.
             llm: The language model to use for answering questions. Default - OpenAI text-davinci-003.
             summary_llm: The language model to use for summarizing documents. If None, llm is used.
+            name: The name of the collection.
+            index_path: The path to the index file IF pickled. If None, defaults to using name in $HOME/.paperqa/name
         """
         self.docs = dict()
         self.chunk_size_limit = chunk_size_limit
@@ -56,9 +61,21 @@ class Docs:
             llm = OpenAI(temperature=0.1)
         if summary_llm is None:
             summary_llm = llm
+        self.update_llm(llm, summary_llm)
+        if index_path is None:
+            index_path = Path.home() / ".paperqa" / name
+        self.index_path = index_path
+
+    def update_llm(self, llm: LLM, summary_llm: Optional[LLM] = None) -> None:
+        """Update the LLM for answering questions."""
+        self.llm = llm
+        if summary_llm is None:
+            summary_llm = llm
+        self.summary_llm = summary_llm
         self.summary_chain = LLMChain(prompt=summary_prompt, llm=summary_llm)
         self.qa_chain = LLMChain(prompt=qa_prompt, llm=llm)
         self.edit_chain = LLMChain(prompt=edit_prompt, llm=llm)
+        self.search_chain = LLMChain(prompt=search_prompt, llm=llm)
 
     def add(
         self,
@@ -105,18 +122,32 @@ class Docs:
         if self._faiss_index is not None:
             self._faiss_index.add_texts(texts, metadatas=metadata)
 
+    @property
+    def doc_previews(self) -> List[Tuple[int, str, str]]:
+        """Return a list of tuples of (key, citation) for each document."""
+        return [
+            (i, doc["metadata"][0]["dockey"], doc["metadata"][0]["citation"])
+            for i, doc in enumerate(self.docs.values())
+        ]
+
     # to pickle, we have to save the index as a file
     def __getstate__(self):
         if self._faiss_index is None:
             self._build_faiss_index()
         state = self.__dict__.copy()
-        state["_faiss_index"].save_local("faiss_index")
+        state["_faiss_index"].save_local(self.index_path)
         del state["_faiss_index"]
+        # remove LLMs (they can have callbacks, which can't be pickled)
+        del state["summary_chain"]
+        del state["qa_chain"]
+        del state["edit_chain"]
+        del state["search_chain"]
         return state
 
     def __setstate__(self, state):
         self.__dict__.update(state)
-        self._faiss_index = FAISS.load_local("faiss_index", OpenAIEmbeddings())
+        self._faiss_index = FAISS.load_local(self.index_path, OpenAIEmbeddings())
+        self.update_llm(OpenAI(temperature=0.1), OpenAI(temperature=0.1))
 
     def _build_faiss_index(self):
         if self._faiss_index is None:
@@ -130,7 +161,18 @@ class Docs:
                 texts, OpenAIEmbeddings(), metadatas=metadatas
             )
 
-    def get_evidence(self, question: str, k: int = 3, max_sources: int = 5):
+    def get_evidence(
+        self,
+        question: str,
+        k: int = 3,
+        max_sources: int = 5,
+        progress: Callable[[str], str] = None,
+    ) -> str:
+        if progress is None:
+
+            def progress(x):
+                return x
+
         context = []
         if self._faiss_index is None:
             self._build_faiss_index()
@@ -146,6 +188,8 @@ class Docs:
                 doc.page_content,
             )
             if "Not applicable" not in c[-1]:
+                print("Here with ", progress)
+                progress(f"Found evidence in {c[1]}\n\n{c[2]}")
                 context.append(c)
             if len(context) == max_sources:
                 break
@@ -157,19 +201,34 @@ class Docs:
             context_str += "\n\nValid keys: " + ", ".join(valid_keys)
         return context_str, context
 
+    def generate_search_query(self, query: str) -> List[str]:
+        """Generate a list of search strings that can be used to find
+        relevant papers.
+
+        Args:
+            query (str): The query to generate search strings for.
+        """
+
+        search_query = self.search_chain.run(question=query)
+        queries = [s for s in search_query.split("\n") if len(s) > 3]
+        # remove 2., 3. from queries
+        queries = [re.sub(r"^\d+\.\s*", "", q) for q in queries]
+        return queries
+
     def query(
         self,
         query: str,
-        k: int = 5,
+        k: int = 10,
         max_sources: int = 5,
         length_prompt: str = "about 100 words",
+        progress: Callable[[str], str] = None,
     ):
         if k < max_sources:
             raise ValueError("k should be greater than max_sources")
         tokens = 0
         with get_openai_callback() as cb:
             context_str, citations = self.get_evidence(
-                query, k=k, max_sources=max_sources
+                query, k=k, max_sources=max_sources, progress=progress
             )
             tokens += cb.total_tokens
         bib = dict()
