@@ -40,3 +40,13 @@ edit_prompt = prompts.PromptTemplate(
     "Part of it may be truncated, please edit the answer to make it complete. "
     "If it appears to be complete, repeat it unchanged.\n\n",
 )
+
+
+search_prompt = prompts.PromptTemplate(
+    input_variables=["question"],
+    template="We want to answer the following question: {question} \n"
+    "Provide three different targeted keyword searches (one search per line) "
+    "that will find papers that help answer the question. Do not use boolean operators. "
+    "The current year is 2023.\n\n"
+    "1.",
+)
